﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceProcess;
using Castle.Windsor;
using App.Server.Initialization;
using App.Server.Services.Core;
using System.Configuration;

namespace App.Server {
    public interface IBootstrapper {
        void Initialize();
        void Run();
    }
    public class Bootstrapper {

        private readonly WindsorContainer _container;
        private readonly ApplicationModeProvider _applicationModeProvider;

        public Bootstrapper() {
            _container = new WindsorContainer();
        }

        public void Initialize() {
            InitializeLogging();
            InstallComponents();
        }

        private void InitializeLogging() {
            log4net.Config.XmlConfigurator.Configure(new System.IO.FileInfo("log4net.config.xml"));
        }

        private void InstallComponents() {
            _container.Install(new ContainerInstaller());
        }

        public void Run() {
            var mode = _applicationModeProvider.ConfigApplicationMode;
            if (mode == ApplicationMode.ConsoleHostService)
                HostWcfServices();
            else 
                HostWindowsService();
        }

        private void HostWcfServices() {
            var hostingServices = _container.Resolve<IHostingServices>();
            hostingServices.Host();

            Console.WriteLine("Press [Enter] to close application..");
            Console.ReadLine();
        }

        private void HostWindowsService() {
            ServiceBase[] ServicesToRun;
            ServicesToRun = new ServiceBase[] 
			{ 
				_container.Resolve<WindowsService>()
			};
            ServiceBase.Run(ServicesToRun);
        }
    }
}
