﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace App.Server.Services.Core {
    public interface IWcfServiceProvider : IDisposable {
        void HostService<TServiceInterface,KServiceImplementation>(string applicationServicesUri) where KServiceImplementation:TServiceInterface;
    }

    public class WcfServiceProvider : IWcfServiceProvider {
        private List<ServiceHost> _services;
        private readonly object _synch = new object();
        private bool _disposed;
        
        public WcfServiceProvider() {
            _services = new List<ServiceHost>();
        }

        public void HostService<TServiceInterface,KServiceImplementation>(string applicationServicesUri) where KServiceImplementation:TServiceInterface {
            CheckIfDisposed();

            lock (_synch) {
                Uri baseAddr = new Uri(applicationServicesUri);
                ServiceHost localHost = new ServiceHost(typeof(KServiceImplementation), baseAddr);

                try {
                    var serviceName = typeof(KServiceImplementation).FullName;
                    localHost.AddServiceEndpoint(typeof(TServiceInterface), new BasicHttpBinding(), serviceName);

                    ServiceMetadataBehavior smb = new ServiceMetadataBehavior();
                    smb.HttpGetEnabled = true;
                    localHost.Description.Behaviors.Add(smb);

                    localHost.Open();

                    _services.Add(localHost);
                } catch (CommunicationException ex) {
                    Console.WriteLine("Oops! Exception: {0}", ex.Message);
                    localHost.Abort();
                }
            }
        }

        private void CheckIfDisposed() {
            if (_disposed)
                throw new ObjectDisposedException("Object disposed");
        }

        public void Dispose() {
            if (_disposed)
                return;

            lock (_synch) {
                foreach (var service in _services) {
                    service.Abort();
                }
                _disposed = true;
            }
        }
    }
}
