﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;
using log4net;
using System.Diagnostics;

namespace Sandpit.Tests {
    
    public class LoggerTest {
        private static ILog _log = LogManager.GetLogger(typeof(LoggerTest));

        [SetUp]
        public void Initialize() {
            log4net.Config.XmlConfigurator.Configure(new System.IO.FileInfo("log4net.config.xml"));
        }
        [Test]
        public void InfoTest() {
            var stopwatch = new Stopwatch();
            _log.DebugFormat("## Performance test started");
            _log.DebugFormat("## DelegateStaticLogger with local cache");
            var numberOfLaps = 1000;

            var cacheTime = (double)LogCounterTest(stopwatch, () => {
                for (int i = 0; i < numberOfLaps; i++)
                    DelegateStaticLogger.InfoCache("Hejka"+i);
            });

            _log.DebugFormat("## DelegateStaticLogger without cache");

            var noCacheTime = (double)LogCounterTest(stopwatch, () => {
                for (int i = 0; i < numberOfLaps; i++)
                    DelegateStaticLogger.InfoNoCache("Hejka" + i);
            });

            _log.DebugFormat("## Normal use ILog");

            var normalLogTime = (double)LogCounterTest(stopwatch, () => {
                for (int i = 0; i < numberOfLaps; i++)
                    _log.Info("Hejka"+i);
            });

            _log.InfoFormat(@"## Test result:
Cache is worse: {0}%
No cache is worse: {1}%", Math.Round((cacheTime - normalLogTime) / normalLogTime * 100,0), Math.Round((noCacheTime - normalLogTime) / normalLogTime * 100,0));
        }

        private long LogCounterTest(Stopwatch stopwatch, Action logAction) {
            stopwatch.Reset();
            stopwatch.Start();

            logAction();
            
            stopwatch.Stop();
            
            var result = stopwatch.ElapsedMilliseconds;

            _log.DebugFormat("## Performance test result {0} ms", result.ToString());
            return result;
        }
    }
}
